use config::{Config, ConfigError, Environment, File};
use dotenvy::dotenv;
use serde::Deserialize;
use std::env;

#[derive(Debug, Deserialize, Clone)]
pub struct AppConfig {
    pub loki_endpoint: String,
    pub service_name: String,
    pub service_version: String,
    pub log_level: String,
    pub port: u16,
    pub environment: String,
}

impl AppConfig {
    pub fn new() -> Result<Self, ConfigError> {
        let _ = dotenv().ok();

        let environment = env::var("APP_ENV").unwrap_or_else(|_| "development".into());

        let config = Config::builder()
            // Start with default values
            .set_default("environment", &*environment)?
            .set_default("service_name", "OpenLayersHub")?
            .set_default("service_version", env!("GIT_VERSION"))?
            .set_default("log_level", "info")?
            .set_default("port", 3000)?
            // Add environment-specific config file
            .add_source(File::with_name(&format!("config/{}", environment)).required(false))
            .add_source(File::with_name(".env").required(false))
            .add_source(
                Environment::with_prefix("APP")
                    .prefix_separator("_")
                    .separator("__"),
            )
            .build()?;

        config.try_deserialize()
    }

    pub fn validate(&self) -> Result<(), Vec<String>> {
        let mut errors = Vec::new();

        if self.loki_endpoint.is_empty() {
            errors.push("Loki endpoint must be set".to_string());
        }

        // Validate environment value
        if !["development", "staging", "production"].contains(&self.environment.as_str()) {
            errors.push(format!(
                "Invalid environment '{}'. Must be one of: development, staging, production",
                self.environment
            ));
        }

        if errors.is_empty() {
            Ok(())
        } else {
            Err(errors)
        }
    }
}
